/*
	Aug 20, 2020:
	This is similar to the baseline threshold estimation, extended to a panel setting.
	This code uses Kremer et al (2013) orthogonal projections transformation to implement the 
	threshold estimation.
	
*/

args inflvar threshvar pcsws_ctrls controls factors expvars sample sample_count boot transform

set matsize 11000
keep if `sample'
local percentile 10
set seed 10
local tex_fmt %4.2f 


*Convert hyphenated varlist to fleshed out varlist in local, only if it is non empty
if "`expvars'"!="" {
	ds `expvars'
	loc expvars `r(varlist)'
}
di "`expvars'"

*Create Labels for Table 
loc lab_exp_infl_MI "Michigan Expectations"
loc lab_exp_infl_SPF_gdp "SPF Expectations"
loc lab_l1_`inflvar' "Lag 1"
loc lab_l2_`inflvar' "Lag 2"
loc lab_l3_`inflvar' "Lag 3"
loc lab_l4_`inflvar' "Lag 4"
loc lab_l5_`inflvar' "Lag 5"


* Simplify varnames into better labels
if "`inflvar'"=="infl_gdp" local infl_label "GDP Deflator"
if "`inflvar'"=="infl_pce_core" local infl_label "PCE Core"
if "`inflvar'"=="infl_cpi_core" local infl_label "CPI Core"
if "`inflvar'"=="infl_pce" local infl_label "PCE"
if "`inflvar'"=="infl_cpi" local infl_label "CPI"
if "`threshvar'"=="u_rate" local thresh_label "Unemployment Rate"


* restrict dataset to regression sample
reg `inflvar' `expvars' `pcsws_ctrls' `controls'
gen tempsample = e(sample)
keep if tempsample == 1
drop tempsample


*** Get panel dimensions (assumes strongly balanced)
ereplace area = group(area)
qui summ area
scalar numID = r(max)
scalar numT = `=_N'/numID

if "`threshvar' `diffur'"!= "`pcsws_ctrls'"  {
	if "`threshvar' `diffur'"!= "`pcsws_ctrls' " { //extra space might throw Stata off (coming from empty local)
		di "`threshvar'" "`diffur'"
		di "`pcsws_ctrls'"
		di ("`threshvar'`diffur'"=="`pcsws_ctrls'")
		di "Error in pcsws_ctrls specification"
		stop //to catch mistake in code
	}
}		


if transform==1{
	* transform each variable:
	by area: gen it=_n
	foreach vv in `inflvar' `expvars' `pcsws_ctrls' `controls' {
		forval n=1/`=numID' {
			forval t=1/`=numT' {
				qui summ F`t'.`vv' if area == `n'
				replace `vv' = (`vv' - r(mean))*sqrt((numT-`t')/(numT-`t'+1)) if it==`t' & area == `n'
			}
		}
	}
}

cap log close

// create factor variables
xi `factors'

* Estimate 1 threshold, then 2 thresholds; compute F-stat - 
qui { // Display only counter & results
	* initialize SSR & gamma scalars for 2 -regime grid search
	scalar rm_`inflvar'_2 = 100000
	scalar gstar_`inflvar'_2 = -666
	
	constraint drop _all
	
	reg `inflvar' `expvars' `pcsws_ctrls' `controls' _I*, vce(cl area)

	local F_lin_`inflvar' = e(rss)

	local obs_n_`inflvar' `=e(N)'
	scalar b_lin = _b[`threshvar']
	scalar SE_lin = _se[`threshvar']
	foreach var in `expvars' {
		scalar b_lin_`var' = _b[`var']
		scalar SE_lin_`var' = _se[`var']
	}
	*Collect coefficients, t-tests, and F-tests for linear
	eststo lintemp
	
	*Assign stars to coefficients
	foreach var in `expvars' `pcsws_ctrls' `controls' {
		test `var'=0
		if r(p)<=.10 & r(p)>.05 local b_lin_`var'_star "*"
		else if r(p)<=.05 & r(p)>.01 local b_lin_`var'_star "**"
		else if r(p)<=.01  local b_lin_`var'_star "***"
		else local b_lin_`var'_star 
	}	
	
	qui levelsof `threshvar' , local(u_local)
		
	
	* Setting bounds for threshold (this will be set for the rest of the file)
	qui sum `threshvar' , detail 
	scalar thresh_`percentile' = r(p`percentile') 
	scalar thresh_`=100-`percentile'' = r(p`=100-`percentile'')
	scalar thresh_spread = floor((`percentile'/100)*e(N))
	
	constraint drop _all
	
	* 2 distinct regime model
	// Create constant
	cap drop ones
	gen ones = 1 
	
	
	local gamma_star_count 1
	foreach gamma_star in `u_local' {

			if (`gamma_star' >= thresh_`percentile') & (`gamma_star' <= thresh_`=100-`percentile'')  { //OB bounds for threshold values (adjustable by percentile)

				di "Counter ....."
				di `gamma_star_count'
				
				preserve

				
				gen left = .
				replace left = 1 if `threshvar'<=`gamma_star'
				replace left = 0 if mi(left)

				gen right = .
				replace right = 1 if !(`threshvar'<=`gamma_star')
				replace right = 0 if mi(right)
			
				constraint drop 1
				constraint 1 `gamma_star'*(`threshvar'_left-`threshvar'_right) + (ones_left-ones_right) = 0
				
				* create constant & u_rate (contemp & lagged diff) for each state
				gen ones_left = ones*left
				gen ones_right = ones*right
				
				local diffur_split
				
				foreach vv in `threshvar' `diffur' {
					gen `vv'_left = `vv'*left
					gen `vv'_right = `vv'*right
				}
				
				// Estimate
				cnsreg `inflvar' `expvars' *_left *_right `controls' _I*,  constraints(1) vce(cl area) nocons collinear
				
				// Collect slope estimates
				scalar bL_2 = _b[`threshvar'_left]
				scalar SE_L_2 = _se[`threshvar'_left]
				scalar bR_2 = _b[`threshvar'_right]
				scalar SE_R_2 = _se[`threshvar'_right]
		
			
				// Minimization for cnsreg
				if ((e(rmse)^2)*e(df_r))<rm_`inflvar'_2 { //cnsreg doesn't report e(rss), so it must be calculated as (root mean squared error)^2 * (residual degrees of freedom)
					eststo minrss //save table for final output
					loc n_obs_2 = `=e(N)'
					scalar rm_`inflvar'_2 =((e(rmse)^2)*e(df_r))
					scalar gstar_`inflvar'_2 = `gamma_star'
					scalar betaL_2 = bL_2
					*local betaL_2_star "``threshvar'_left_star'"
					scalar stderrL_2 = SE_L_2
					scalar betaR_2 = bR_2
					*local betaR_2_star "``threshvar'_right_star'"
					scalar stderrR_2 = SE_R_2	
					foreach var in `expvars' {
						scalar b_thr_`var' = _b[`var']
						*local b_thr_`var'_star "``var'_star'"
						scalar SE_thr_`var' = _se[`var']
					}	
					
					* R-sq
					qui summ `inflvar', d
					loc Rsq `= 1 - (rm_`inflvar'_2 /(r(Var)*`n_obs_2'))'

					
					* F stat for LR test
					local F_`inflvar'_est  `=`obs_n_`inflvar''*(`F_lin_`inflvar'' - rm_`inflvar'_2)/rm_`inflvar'_2'

					noi di "2 Distinct Regimes, `inflvar' Sample: `sample', `gamma_star_count'"

					*collect stars
					foreach var of varlist `threshvar'_* `expvars' {
						test `var'=0
						if r(p)<=.10 & r(p)>.05 local `var'_star "*"
						else if r(p)<=.05 & r(p)>.01 local `var'_star "**"
						else if r(p)<=.01  local `var'_star "***"
						else local `var'_star 
					}						
					
					eststo t_temp //save table for final output
					
					*Pseudo-predict 1-threshold model
					foreach var of varlist `expvars' `controls' _I* { 
						di "`var'"
						qui sum `var'
						replace `var' = r(mean)
					} 				
					
					predict pred_`inflvar'_1thr
					keep halfyear area pred_`inflvar'_1thr
					
					tempfile pred_1thr //to merge "final" predicted value into dataset for graphing
					save `pred_1thr', replace
	
					
				} 
				
				else {
					di ((e(rmse)^2)*e(df_r))
					
				}
				
				// drop coefficients & re-estimate
				restore
			}
			
		local gamma_star_count `=`gamma_star_count'+1' //iteration counter	
	} //==> gamma_star loop
	
	
* graph of predicted value versus raw data
merge 1:1 halfyear area using `pred_1thr', nogen
//sort `threshvar'

	
* end of estimation

// Output estimates (all for now, but keep only useful ones later)

/*
*Output results
	log using "../output/Paper/`folder_dir'/1thrBOOT_`inflvar'_`threshvar'_`sample_count'_t`=transform'.txt", text replace
	n di "THRESHOLD ESTIMATION (GRID SEARCH)"
	n di "Date & time of running - `c(current_time)' `c(current_date)' "
	n di "Sample: `sample'"
	n di ""
	n di "Iteration Counter: `counter_`inflvar''" //may be useful later; provides integer markers for iterations rather than datapoints themselves
	n di "Inflation Measure: `inflvar'"
	n di ""
	n di "2 REGIME MODEL (SEs clustered on MSA)"
	n di "-------------------------------------------------"
	n di "Slack Beta (Linear model): " `tex_fmt'  b_lin "`b_lin_`threshvar'_star'"
	n di "SE: " `tex_fmt' SE_lin
	n di ""
	n di "Slack Beta Left : " `tex_fmt' betaL_2 "``threshvar'_left_star'"
	n di "SE: " `tex_fmt' stderrL_2
	n di ""	
	n di "Slack Beta Right : " `tex_fmt' betaR_2 "``threshvar'_right_star'"
	n di "SE: " `tex_fmt' stderrR_2
	n di ""	
	
	foreach var in `expvars' {
		if regexm("`lab_`var''", "Lag") continue
		n di "`lab_`var'' Coef (Linear): " `tex_fmt' b_lin_`var' "`b_lin_`var'_star'"
		n di "SE (Linear): " `tex_fmt' SE_lin_`var' 
		n di ""
		n di "`lab_`var'' Coef (Threshold): " `tex_fmt' b_thr_`var' "``var'_star'"
		n di "SE (Threshold): " `tex_fmt' SE_thr_`var'
		n di ""
	}

	n di ""	
	n di "Threshold Star: " `tex_fmt' gstar_`inflvar'_2
	n di ""
	n di "R-sq.: " `Rsq'
	n di "Number of observations: " `n_obs_2'
	n di "-------------------------------------------------"	
	n di "F-Test: " `tex_fmt' `F_`inflvar'_est'
	n di "Minimum RSS value: " `tex_fmt' rm_`inflvar'_2
	n di "Linear RSS: " `tex_fmt' `F_lin_`inflvar''
	n di ""
	n di "-------------------------------------------------"	
	n di ""
log close
*/
scalar nobs = `=`n_obs_2''
} // end output suppression

* Bootstrap to CIs for params
// Not clustering SEs to save computing time: wno't affect RSS 
if "`boot'" == "Y" {
	
qui { // Display only counter & results

loc gstar = gstar_`inflvar'_2 // estimated threshold pt
loc beta1_star = betaL_2 // estimated slope
loc beta2_star = betaR_2 // estimated slope
loc F_est = `F_`inflvar'_est' // Estimated F stat


* Construct resids taking gammastar as given & linear resids for p value
		cap drop yhat linfits resid linresid ones* left right
		constraint drop _all	

		reg `inflvar' `expvars' `pcsws_ctrls' `controls' _I*
		predict linfits, xb
		predict linresid, res					
		
		gen left = .
		replace left = 1 if `threshvar'<=`gstar'
		replace left = 0 if mi(left)

		gen right = .
		replace right = 1 if !(`threshvar'<=`gstar')
		replace right = 0 if mi(right)
		
		constraint 1 `gstar'*(`threshvar'_left-`threshvar'_right) + (ones_left-ones_right) = 0
		
		* create constant & slack var for each state
		gen ones_left = ones*left
		gen ones_right = ones*right
		
		foreach vv in `threshvar' {
			gen `vv'_left = `vv'*left
			gen `vv'_right = `vv'*right		
		}
		
		// Compute fitted values and residuals from nonlinear model, taking threshold as given
		cnsreg `inflvar' `expvars' *_left *_right `controls' _I*,  constraints(1) nocons collinear
		local obs_n `=e(N)'
		
		predict yhat, xb
		predict resid, res			
	
		// preallocate to store bootstrapped values
		loc n_sim = 250
		matrix Fboot = J(`n_sim',1,.) // to construct CIs for the threshold point
		matrix Fboot_lin = J(`n_sim',1,.) // to construct p-value for test of the null of no threshold
		
		matrix gammaBoot = J(`n_sim',1,.)
		matrix beta1Boot= J(`n_sim',1,.)
		matrix beta2Boot= J(`n_sim',1,.)
		
		preserve
		clear	
		set obs `=numID'
		gen area = _n
		tempfile id
		save `id'
		restore
		
		bys area: gen ct=_n // get obs number by group
		
		forval s = 1/`n_sim' {
			noi di "`inflvar', Sample: `sample', Iteration `s'"
						
			cap drop wt *resid_boot `inflvar'_*boot
			
			gen wt = floor(2*runiform()) if ct==1 // wt for wild bootstrap, at the cluster level
			replace wt = -1 if wt==0

			bys area: carryforward wt, replace
			
			*Transform residuals in wild bootstrap			
			gen resid_boot = resid*wt
			gen linresid_boot = linresid*wt
	
	
			// create resampled y for threshold and linear models
			gen `inflvar'_linboot = linfits+linresid_boot
			gen `inflvar'_boot = yhat+resid_boot
			
			* SSR from linear model w/ resampled y
			reg `inflvar'_linboot `expvars' `pcsws_ctrls' `controls' _I*

			local F_lin_`inflvar'_boot = e(rss)			
			
			* create SSR using resampled y & gammastar
			// using gamma-dependent variables defined earlier
			
			// Compute fitted values and residuals, taking threshold as given
			cnsreg `inflvar'_boot `expvars' *_left *_right `controls' _I*,  constraints(1) nocons collinear	
			*local rss_h0 `=(e(rmse)^2)*e(df_r)' // rss for est model
			
			* initialize SSR & gamma scalars
			scalar rm_`inflvar'_boot = 100000
			scalar rm_`inflvar'_linboot = 100000
			scalar gl_`inflvar'_boot = -666
			scalar gu_`inflvar'_boot = -666
			scalar bL_boot = -666
			scalar bR_boot = -666
					
			qui sum `threshvar' , detail 
			qui levelsof `threshvar' , local(u_local)
			
			* Grid search using resampled y
			// Create constant
			cap drop ones
			gen ones = 1 
			
			local gamma_star_count_2 1
			foreach gamma_star in `u_local' {
			if (`gamma_star' >= thresh_`percentile') & (`gamma_star' <= thresh_`=100-`percentile'')  { //OB bounds for threshold values (adjustable by percentile)
			
					
					constraint drop 1
					di "Counter ....."
					di `gamma_star_count'
				
					
					preserve
					
					cap drop *left *right
					
					
					gen left = .
					replace left = 1 if `threshvar'<=`gamma_star'
					replace left = 0 if mi(left)

					gen right = .
					replace right = 1 if !(`threshvar'<=`gamma_star')
					replace right = 0 if mi(right)
						
					* create constant & u_rate (contemp & lagged diff) for each state
					gen ones_left = ones*left
					gen ones_right = ones*right
			
					
					foreach vv in `threshvar' `diffur' {
						gen `vv'_left = `vv'*left
						gen `vv'_right = `vv'*right	
					}
						
						
						
						
					* Define constraint
					constraint 1 `gamma_star'*(`threshvar'_left-`threshvar'_right) + (ones_left-ones_right) = 0
					cnsreg `inflvar'_boot `expvars' *_left *_right `controls' _I*,  constraints(1) nocons collinear	
					
					// Collect slope estimates
					scalar bL_2 = _b[`threshvar'_left]
					scalar bR_2 = _b[`threshvar'_right]				
					
					// Minimization for cnsreg; save F stat, gamma and slopes -- null is a threshold model
					if ((e(rmse)^2)*e(df_r))<rm_`inflvar'_boot { //cnsreg doesn't report e(rss), so it must be calculated as (root mean squared error)^2 * (residual degrees of freedom)
						
						scalar bL_boot = bL_2
						scalar bR_boot = bR_2
						scalar rm_`inflvar'_boot =((e(rmse)^2)*e(df_r))
						scalar gstar_`inflvar'_boot = `gamma_star'					
						local counter_`inflvar'  "`gamma_star_count_2'"
						//noi di as text "`counter_`inflvar''"
						
						* change variables to reflect estimated gamma star case, then replace
						qui ds *left *right
						foreach vv in `r(varlist)' {
							gen `vv'_keep = `vv'
						}
						
						drop *left *right
					
						gen left = .
						replace left = 1 if `threshvar'<=`gstar'
						replace left = 0 if mi(left)

						gen right = .
						replace right = 1 if !(`threshvar'<=`gstar')
						replace right = 0 if mi(right)
						
						* create constant & u_rate (contemp & lagged diff) for each state
						gen ones_left = ones*left
						gen ones_right = ones*right
						
						foreach vv in `threshvar' {
							gen `vv'_left = `vv'*left
							gen `vv'_right = `vv'*right	
						}					
						
						predict e_null, resid
						summ e_null
						loc rss_h0 = `r(Var)'*`obs_n'
						local F_boot  `=`obs_n'*(`rss_h0' - rm_`inflvar'_boot)/rm_`inflvar'_boot'
						
						drop *left *right
						ren *_keep *
						
					}
					else {
						di ((e(rmse)^2)*e(df_r))					
					}

					cnsreg `inflvar'_boot `expvars' *_left *_right `controls' _I*,  constraints(1) nocons collinear
					
					// Null is a linear model
					if ((e(rmse)^2)*e(df_r))<rm_`inflvar'_linboot {
						scalar rm_`inflvar'_linboot =((e(rmse)^2)*e(df_r))
						local F_linboot  `=`obs_n'*(`F_lin_`inflvar'_boot' - rm_`inflvar'_linboot)/rm_`inflvar'_linboot'
					}
					else {
						di ((e(rmse)^2)*e(df_r))					
					}				
					
					// drop coefficients & re-estimate
					restore
					}
					
				local gamma_star_count_2 `=`gamma_star_count_2'+1' //iteration counter	
			} //==> gamma_star loop
			
			* Save the F-stats, slopes and gammas from bootstrapping 
			matrix Fboot[`s',1] = `F_boot'
			matrix Fboot_lin[`s',1] = `F_linboot'
			matrix gammaBoot[`s',1] = gstar_`inflvar'_boot
			matrix beta1Boot[`s',1] =bL_boot
			matrix beta2Boot[`s',1] =bR_boot
			
	} // over bootstrap draws


*Generate bootstrapped CIs
preserve
foreach mm in Fboot Fboot_lin gammaBoot beta1Boot beta2Boot {
	svmat `mm', names(`mm')
}

// Create p-values for existence of threshold
count if Fboot_lin1 >= `F_est' & !mi(Fboot_lin1)
scalar pval = `r(N)'/`n_sim'


// create CIs for slopes
gen delta_1 = abs(beta1Boot1-`beta1_star')
gen delta_2 = abs(beta2Boot1-`beta2_star')

forval i=1/2 {
	qui summ delta_`i', d
	loc q_`i' = `r(p95)'
	scalar beta`i'_Lci = `beta`i'_star' - `q_`i''
	scalar beta`i'_Uci = `beta`i'_star' + `q_`i''
}

// for gamma
summ Fboot1, d
// keep gammas for which the F stat is below the critical value
loc crit_F = `r(p95)'
gen gCI_set = gammaBoot1 if Fboot1 <= `crit_F'
summ gCI_set, d
scalar gamma_Lci = `r(min)'
scalar gamma_Uci = `r(max)'

restore
/*
log using "../output/Paper/`folder_dir'/1thrBOOT_`inflvar'_`threshvar'_`sample_count'_t`=transform'.txt", text append
	
n di "Bootstrapped p-val and CIs [`n_sim' replications]"
n di "--------------------------------------------------------------------"
n di "Date & time of running - `c(current_time)' `c(current_date)' "
n di "Sample: `sample'"
n di ""
n di "Inflation Measure: `inflvar'"
n di ""
n di "p-value for Gamma: " pval
n di "Max Gamma: " gamma_Uci
n di "Min Gamma: " gamma_Lci
n di ""
n di "Beta 1; Upper Bound: " beta1_Uci
n di "Beta 1; Lower Bound: " beta1_Lci
n di " "
n di "Beta 2; Upper Bound: " beta2_Uci
n di "Beta 2; Lower Bound: " beta2_Lci
n di " "
log close
*/
} // end output suppression
}

