/*
	This file runs an IV regression on our threshold model after fixing the threshold. Run this do file out of IVreg_hansen.do	
*/

/*
	May 31, 2018: * Significance stars reflect NW SEs, unless otherwise noted
	
	1 Threshold, bootstrap CIs for gamma and betas using the Wild Bootstrap in Hansen (2015)
	This file also reports the F-stats from the subsamples considered in the paper
	Incorporates significance stars and other elements for final table output
*/

/* 
	Syntax for calling this do file:
	do "`do_dir'/thresholdtests_2_A_1thr" infl_pce_core u_rate "``var'_pcsws_ctrls'" "``var'_controls'" "`expvars'" "`sample'" "`sample_count'"
*/

args inflvar threshvar pcsws_ctrls controls expvars sample instrspec sample_count boot
di "`expvars'"
set matsize 11000
keep if `sample'
local percentile 10
set seed 10
local tex_fmt %4.2f 
di "`pcsws_ctrls'"

***Define instrument here
loc instr_RR RRshocks_`instrspec' //either Fed Funds or shadow rate instruments

*Lag instruments and put in local
loc instrlags
forval i=0/20 {
	gen l`i'_`instr_RR' = L`i'.`instr_RR'
	loc instrlags "`instrlags' l`i'_`instr_RR'"
}


*Convert hyphenated varlist to fleshed out varlist in local
ds `expvars'
loc expvars `r(varlist)'
di "`expvars'"

*Create Labels for Table 
loc lab_exp_infl_MI "Michigan Expectations"
loc lab_exp_infl_SPF_gdp "SPF Expectations"
loc lab_l1_`inflvar' "Lag 1"
loc lab_l2_`inflvar' "Lag 2"
loc lab_l3_`inflvar' "Lag 3"
loc lab_l4_`inflvar' "Lag 4"
loc lab_l5_`inflvar' "Lag 5"

*Period 1 (Oil Shock Inflation)
loc period_1_start `=q(1976q2)'
loc period_1_end `=q(1980q1)'

*Period 2 (Volcker)
loc period_2_start `=q(1980q2)'
loc period_2_end `=q(1985q4)'

*Period 3 (Great Recession)
loc period_3_start `=q(2009q1)'
loc period_3_end `=q(2013q4)'

count if quarter>=`period_1_start' & quarter<=`period_1_end'
loc obs_n1_`inflvar' `=r(N)' //observations in period 1
count if quarter>=`period_2_start' & quarter<=`period_2_end'
loc obs_n2_`inflvar' `=r(N)' //observations in period 2
count if quarter>=`period_3_start' & quarter<=`period_3_end'
loc obs_n3_`inflvar' `=r(N)' //observations in period 3
di `obs_n3_`inflvar''


/*
//check if the args passed successfully 
di "`inflvar'"
di "`threshvar'"
di "`pcsws_ctrls'"
di "`controls'"
di "`expvars'"
di "`sample'"
di "`sample_count'"
di "`boot'" 
di "`LR'"  
*/

* Simplify varnames into better labels
if "`inflvar'"=="infl_gdp" local infl_label "GDP Deflator"
if "`inflvar'"=="infl_pce_core" local infl_label "PCE Core"
if "`inflvar'"=="infl_cpi_core" local infl_label "CPI Core"
if "`inflvar'"=="infl_pce" local infl_label "PCE"
if "`inflvar'"=="infl_cpi" local infl_label "CPI"
if "`threshvar'"=="labor_share" local thresh_label "Labor Share"
if "`threshvar'"=="u_gap" local thresh_label "Unemployment Gap"
if "`threshvar'"=="u_rate" local thresh_label "Unemployment Rate"

* restrict dataset to regression sample
qui reg `inflvar' `expvars' `pcsws_ctrls' `controls'
gen tempsample = e(sample)
keep if tempsample == 1
drop tempsample

* Fix threshold
if "`inflvar'"=="infl_cpi" & "`threshvar'"=="u_gap" loc thresh_fix 1.95 //

* Lag threshold variable or not
loc lthr L. //


**Keep data for Matlab
/*
preserve
keep quarter `inflvar' `threshvar' `expvars'  `controls' `instrlags' 
order quarter `inflvar' `threshvar' `expvars'  `controls' `instrlags'
qui reg `inflvar' `threshvar' `expvars'  `controls' `instrlags' 
gen tempsample = e(sample)
keep if tempsample == 1
drop tempsample
export excel using "BM_matlab_ourdata.xlsx", replace
restore
*/

* In case we decide to include difference of urate in regression, then it needs to be averaged for graph
if "`threshvar'"!="`pcsws_ctrls'" {
	if "`inflvar'"=="infl_gdp" {
		qui ds l1_d_u_rate - l4_d_u_rate
		local diffur "`r(varlist)'"
	}	
	else {
		qui ds l0_d_u_rate - l2_d_u_rate
		local diffur "`r(varlist)'"	
	}
}

else {
	local diffur //otherwise leave blank
}	

if "`threshvar' `diffur'"!= "`pcsws_ctrls'"  {
	if "`threshvar' `diffur'"!= "`pcsws_ctrls' " { //extra space might throw Stata off (coming from empty local)
		di "`threshvar'" "`diffur'"
		di "`pcsws_ctrls'"
		di ("`threshvar'`diffur'"=="`pcsws_ctrls'")
		di "Error in pcsws_ctrls specification"
		stop //to catch mistake in code
	}
}		

* Create expecatations constraint
qui ds `expvars'

scalar n_expvars = `=wordcount("`r(varlist)'")'
loc expdesc "No lagged inflation" // for title
if n_expvars > 1 { // if more than 1 inflation expectations term
	loc expectations `=subinstr("`r(varlist)'"," ","+",.)' = 1
	loc expdesc "Lagged inflation included" // overwrite
	noi di n_expvars
	di "loc cnstr_var : word `=n_expvars' of `expvars'" //choose some variable to subtract out to imitate constraint for Newey
	loc cnstr_var : word `=n_expvars' of `expvars' //choose some variable to subtract out to imitate constraint for Newey
	noi di "`cnstr_var'"
}	

*Generate constrained variables of constrained regression
cap drop *_cns
ds `expvars'
foreach var in `r(varlist)' {
	if `var'!=`cnstr_var' {
		gen `var'_cns = `var' - `cnstr_var' //subtract out first variable of expvars from dependent variable and expectation variables
	}
}
ds *_cns
loc expvars_cns `r(varlist)'
gen `inflvar'_cns = `inflvar' - `cnstr_var'

*di "`expectations'"

cap log close



* Estimate 1 threshold, then 2 thresholds; compute F-stat - 
qui { // Display only counter & results
	* initialize SSR & gamma scalars for 2 -regime grid search
	scalar rm_`inflvar'_2 = 100000
	scalar gstar_2 = -666
	
	constraint drop _all
	// Constrain sum of inflation expectations to 1
	constraint 1 `expectations'
	
	/*
	if n_expvars > 1 {
		cnsreg `inflvar' `expvars' `pcsws_ctrls' `controls' , constraints(1)
	}
	else {
		reg `inflvar' `expvars' `pcsws_ctrls' `controls' // reg still saves rmse & df_r in e()
	}
	*/
	
	
	****Newey West and robust IV regression (Manually coded constraints at top of code)
	ivregress 2sls `inflvar'_cns `expvars_cns' (`threshvar'=`instrlags') `controls', vce(robust)
	//esttab using "../output/Paper/`folder_dir'/IVlin_`inflvar'_`threshvar'_base_rob.xls", replace r2 se star(* 0.10 ** 0.05 *** 0.01)
	newey2 `inflvar'_cns `expvars_cns' (`threshvar'=`instrlags') `controls', lag(5)
	eststo lin_`instrspec' //store linear estimate

	******Run OLS on IV sample above
	eststo ols_lin_IVsamp: newey `inflvar'_cns `expvars_cns' `pcsws_ctrls' `controls' if e(sample), lag(5) force //e(sample) is IV sample above
	
			
	/*
	local F_lin_`inflvar' `=(e(rmse)^2)*e(df_r)' // cnsreg doesn't save e(rss)
	
	**Collect subsample RSS for linear model
	predict lin_resid, res //collect residuals in variable
	replace lin_resid = lin_resid^2
	//collect RSS in of 3 different subsamples in 3 different variables
	egen lin_resid1 = sum(lin_resid) if quarter>=`period_1_start' & quarter<=`period_1_end'
	egen lin_resid2 = sum(lin_resid) if quarter>=`period_2_start' & quarter<=`period_2_end' 
	egen lin_resid3 = sum(lin_resid) if quarter>=`period_3_start' & quarter<=`period_3_end'
	forval i=1/3 { //RSS = max = mean = min for each variable ==> place number into a local
		sum lin_resid`i'
		loc F`i'_lin_`inflvar' `=r(mean)'
	}	
	drop lin_resid
	
	local obs_n_`inflvar' `=e(N)'
	scalar b_lin = _b[`threshvar']
	scalar SE_lin = _se[`threshvar']
	foreach var in `expvars' {
		scalar b_lin_`var' = _b[`var']
		scalar SE_lin_`var' = _se[`var']
	}
	*Collect coefficients, t-tests, and F-tests for linear
	eststo lintemp
	esttab lintemp using "../output/Paper/`folder_dir'/lineartest_`inflvar'_`threshvar'_base.txt", replace r2 se star(* 0.10 ** 0.05 *** 0.01)

	lincom l1_`inflvar' + l2_`inflvar' + l3_`inflvar' + l4_`inflvar' + l5_`inflvar'
	scalar lin_lagsum_coef = r(estimate)
	scalar lin_lagsum_se = r(se)
	test l1_`inflvar' + l2_`inflvar' + l3_`inflvar' + l4_`inflvar' + l5_`inflvar' = 0
	//scalar lin_lagsum_p = r(p)

	if r(p)<=.10 & r(p)>.05 local lin_lagsum_coef_star "*"
	else if r(p)<=.05 & r(p)>.01 local lin_lagsum_coef_star "**"
	else if r(p)<=.01  local lin_lagsum_coef_star "***"
	else local lin_lagsum_coef_star 
	
	*Newey West
	
	preserve
	cap drop *_cns
	ds `expvars'
	foreach var in `r(varlist)' {
		if `var'!=`cnstr_var' {
			gen `var'_cns = `var' - `cnstr_var' //subtract out first variable of expvars from dependent variable and expectation variables
		}
	}
	ds *_cns
	loc expvars_cns `r(varlist)'
	gen `inflvar'_cns = `inflvar' - `cnstr_var'
	newey `inflvar'_cns `expvars_cns' `pcsws_ctrls' `controls', lag(5)
	
	*Assign stars to coefficients
	foreach var in `threshvar' `expvars_cns' {
		test `var'=0
		if r(p)<=.10 & r(p)>.05 local b_lin_`var'_star "*"
		else if r(p)<=.05 & r(p)>.01 local b_lin_`var'_star "**"
		else if r(p)<=.01  local b_lin_`var'_star "***"
		else local b_lin_`var'_star 
	}

	eststo nw_lintemp
	esttab nw_lintemp using "../output/Paper/`folder_dir'/nw_lineartest_`inflvar'_`threshvar'_base.txt", replace r2 se star(* 0.10 ** 0.05 *** 0.01)
	scalar nw_SE_lin = _se[`threshvar']
	foreach var in `expvars_cns' {
		scalar nw_lin_`var' = _se[`var']
	}
	restore
	*/
	qui levelsof `threshvar' , local(u_local)
		
	
	* Setting bounds for threshold (this will be set for the rest of the file)
	qui sum `threshvar' , detail 
	scalar thresh_`percentile' = r(p`percentile') 
	scalar thresh_`=100-`percentile'' = r(p`=100-`percentile'')
	scalar thresh_spread = floor((`percentile'/100)*e(N))
	
	constraint drop _all
	// Constrain sum of inflation expectations to 1
	constraint 1 `expectations'	
	
	* 2 distinct regime model
	// Create constant
	cap drop ones
	gen ones = 1 
	
	
	local gamma_star_count 1
	foreach gamma_star in `u_local' `thresh_fix' { //add thresh_fix to grid search (at the end)
	
			if (`gamma_star' >= thresh_`percentile') & (`gamma_star' <= thresh_`=100-`percentile'')  { //OB bounds for threshold values (adjustable by percentile)

				di "Counter ....."
				di `gamma_star_count'
				
				preserve

				
				gen left = .
				replace left = 1 if `lthr'`threshvar'<=`gamma_star' //lag the threshold variable 
				replace left = 0 if mi(left) //& !missing(L.`threshvar') //first observation should be missing
				
				
				gen right = .
				replace right = 1 if !(`lthr'`threshvar'<=`gamma_star') //lag the threshold variable
				replace right = 0 if mi(right) //& !missing(L.`threshvar') //first observation should be missing
				
				constraint drop 2
				constraint 2 `gamma_star'*(`threshvar'_left-`threshvar'_right) + (ones_left-ones_right) = 0
				
				gen `threshvar'_left_cns = (`threshvar'-`gamma_star')*left //subtract out gamma
				gen `threshvar'_right_cns = (`threshvar'-`gamma_star')*right //subtract out gamma
				
				* create constant & urate (contemp & lagged diff) for each state
				gen ones_left = ones*left
				gen ones_right = ones*right
				
				local diffur_split
				
				
				foreach vv in `threshvar' /*`diffur'*/ `instrlags' {
					gen `vv'_left = `vv'*left
					gen `vv'_right = `vv'*right
					/*
					if "`vv'"!="`threshvar'" { //for taking the mean 
						local diffur_split `diffur_split' `vv'_left `vv'_right	
					}	
					*/
				}
				
			
				ds l*`instr_RR'_left
				loc instrlags_left `r(varlist)'
				ds l*`instr_RR'_right
				loc instrlags_right `r(varlist)'
				
				// Estimate

				//cnsreg `inflvar' `expvars' *_left *_right `controls' ,  constraints(1-2) nocons collinear
				
				//reg3  (`inflvar' `threshvar'_* ones_*  `expvars' `controls', nocons ) (`threshvar'_left L(0/20).`instr_RR'_left   `expvars' `controls') (`threshvar'_right L(0/20).`instr_RR'_right   `expvars' `controls'), 3sls constraints(1-2)

				eststo opthr: ivregress 2sls `inflvar'_cns `expvars_cns' (`threshvar'_left_cns `threshvar'_right_cns=`instrlags_left'  `instrlags_right') `controls', vce(robust)
				
				**********************************
				*****Save fixed threshold results
				**********************************
				if `gamma_star'==`thresh_fix' {
					
					*IV with fixed threshold
					eststo t_`instrspec'_fixed: newey2 `inflvar'_cns `expvars_cns' (`threshvar'_left_cns `threshvar'_right_cns=`instrlags_left'  `instrlags_right') `controls', lag(5)
					
					scalar nobs_fixed_IV = `=e(N)'
					test `threshvar'_left_cns = `threshvar'_right_cns
					scalar coeftest_c_fixed_IV = r(p) 
					if regexm("`expvars'", "exp_infl_SPF_gdp") & regexm("`expvars'", "exp_infl_MI") {
						lincom exp_infl_SPF_gdp_cns + exp_infl_MI_cns - 1
					
						scalar lagsumNW_coef_fixed_IV = -1*r(estimate)
						scalar lagsumNW_se_fixed_IV = r(se)				
					}

					else {
						n di "Is this model supposed to be lags only?"
						nr
					}
					
					
					
					
					*OLS with fixed threshold
					eststo t_ols_fixed_IVsamp: newey `inflvar'_cns `expvars_cns' `threshvar'_left_cns `threshvar'_right_cns `controls' if e(sample) , lag(5) force //e(sample) is the IV sample size
					scalar nobs_fixed_OLS = `=e(N)'
					test `threshvar'_left_cns = `threshvar'_right_cns
					scalar coeftest_c_fixed_OLS = r(p) 
					if regexm("`expvars'", "exp_infl_SPF_gdp") & regexm("`expvars'", "exp_infl_MI") {
						lincom exp_infl_SPF_gdp_cns + exp_infl_MI_cns - 1
					
						scalar lagsumNW_coef_fixed_OLS = -1*r(estimate)
						scalar lagsumNW_se_fixed_OLS = r(se)	
						
					}

					else {
						n di "Is this model supposed to be lags only?"
						nr
					}
				} //==> end of "if threshold is preset fixed threshold" statement
							
				// Collect slope estimates
				scalar bL_2 = _b[`threshvar'_left]
				scalar SE_L_2 = _se[`threshvar'_left]
				scalar bR_2 = _b[`threshvar'_right]
				scalar SE_R_2 = _se[`threshvar'_right]
		

				// Minimization for cnsreg
				if e(rss)<rm_`inflvar'_2 { //cnsreg doesn't report e(rss), so it must be calculated as (root mean squared error)^2 * (residual degrees of freedom)
					noi di "Threshold: `gamma_star'"
					//esttab opthr using "../output/Paper/`folder_dir'/IVthresh_opt_`inflvar'_`threshvar'_base_rob.xls", replace r2 se star(* 0.10 ** 0.05 *** 0.01)
					eststo minrss //save table for final output
					scalar nobs_opt = `=e(N)'
					scalar rm_`inflvar'_2 = e(rss)
					scalar gstar_2 = `gamma_star'
					scalar betaL_2 = bL_2
					*local betaL_2_star "``threshvar'_left_star'"
					scalar stderrL_2 = SE_L_2
					scalar betaR_2 = bR_2
					*local betaR_2_star "``threshvar'_right_star'"
					scalar stderrR_2 = SE_R_2	

					eststo t_`instrspec'_opt: newey2 `inflvar'_cns `expvars_cns' (`threshvar'_left_cns `threshvar'_right_cns=`instrlags_left'  `instrlags_right') `controls', lag(5)
					
					//esttab optnew using "../output/Paper/`folder_dir'/IVthresh_opt_`inflvar'_`threshvar'_base_newey.xls", replace r2 se star(* 0.10 ** 0.05 *** 0.01)
					test `threshvar'_left_cns = `threshvar'_right_cns
					scalar coeftest_c_opt = r(p) 
					
					
					if regexm("`expvars'", "exp_infl_SPF_gdp") & regexm("`expvars'", "exp_infl_MI") {
						lincom exp_infl_SPF_gdp_cns + exp_infl_MI_cns - 1
					
						scalar lagsumNW_coef_opt = -1*r(estimate)
						scalar lagsumNW_se_opt = r(se)	
						
						scalar z = abs(lagsumNW_coef_opt/lagsumNW_se_opt)

						scalar p = exp((0.743 - (z+0.862)^2)/2.404)
						
						if p < 0.01 {
							loc lagsumNW_coef_star_opt "***"
						}
						else if p >= 0.01 & p < 0.05 {
							loc lagsumNW_coef_star_opt "**"					
						}
						else if p >= 0.05 & p < 0.1 {
							loc lagsumNW_coef_star_opt "*"					
						}						
						else {
							loc lagsumNW_coef_star_opt ""
						}
						
					}

					else {
						n di "Is this model supposed to be lags only?"
						nr
					}
					
					/*
					*F-statistic for sum of lags
					test l1_`inflvar' + l2_`inflvar' + l3_`inflvar' + l4_`inflvar' + l5_`inflvar' = 0
					if r(p)<=.10 & r(p)>.05 local lagsum_coef_star "*"
					else if r(p)<=.05 & r(p)>.01 local lagsum_coef_star "**"
					else if r(p)<=.01  local lagsum_coef_star "***"
					else local lagsum_coef_star 
					lincom l1_`inflvar' + l2_`inflvar' + l3_`inflvar' + l4_`inflvar' + l5_`inflvar'
					scalar lagsum_coef = r(estimate)
					scalar lagsum_se = r(se)
					local F_`inflvar'_est  `=`obs_n_`inflvar''*(`F_lin_`inflvar'' - rm_`inflvar'_2)/rm_`inflvar'_2'
					
					
					* R-sq
					summ `inflvar', d
					loc Rsq `= 1 - (rm_`inflvar'_2 /(r(Var)*`n_obs_2'))'

					*F-statistic for sum of lags
					test l1_`inflvar' + l2_`inflvar' + l3_`inflvar' + l4_`inflvar' + l5_`inflvar' = 0
					if r(p)<=.10 & r(p)>.05 local lagsum_coef_star "*"
					else if r(p)<=.05 & r(p)>.01 local lagsum_coef_star "**"
					else if r(p)<=.01  local lagsum_coef_star "***"
					else local lagsum_coef_star 
					lincom l1_`inflvar' + l2_`inflvar' + l3_`inflvar' + l4_`inflvar' + l5_`inflvar'
					scalar lagsum_coef = r(estimate)
					scalar lagsum_se = r(se)
					local F_`inflvar'_est  `=`obs_n_`inflvar''*(`F_lin_`inflvar'' - rm_`inflvar'_2)/rm_`inflvar'_2'
					
					*F-statistic for subsample periods
					predict thresh_resid, res //collect residuals in variable
					replace thresh_resid = thresh_resid^2
					//collect RSS in of 3 different subsamples in 3 different variables
					egen thresh_resid1 = sum(thresh_resid) if quarter>=`period_1_start' & quarter<=`period_1_end'
					egen thresh_resid2 = sum(thresh_resid) if quarter>=`period_2_start' & quarter<=`period_2_end' 
					egen thresh_resid3 = sum(thresh_resid) if quarter>=`period_3_start' & quarter<=`period_3_end'
					forval i=1/3 { //RSS = max = mean = min for each variable ==> place number into a local
						sum thresh_resid`i'
						loc rm_`inflvar'_sub`i' `=r(mean)'
					}
					local F1_`inflvar'_est  `=`obs_n1_`inflvar''*(`F1_lin_`inflvar'' - `rm_`inflvar'_sub1')/`rm_`inflvar'_sub1''
					local F2_`inflvar'_est  `=`obs_n2_`inflvar''*(`F2_lin_`inflvar'' - `rm_`inflvar'_sub2')/`rm_`inflvar'_sub2''
					local F3_`inflvar'_est  `=`obs_n3_`inflvar''*(`F3_lin_`inflvar'' - `rm_`inflvar'_sub3')/`rm_`inflvar'_sub3''
					
					drop thresh_resid*
					noi di "2 Distinct Regimes, `inflvar' Sample: `sample', `gamma_star_count'"
					
					*Newey-West with Regression Kink Model
					
					cap drop *_cns
					ds `expvars'
					foreach var in `r(varlist)' {
						if `var'!=`cnstr_var' {
							gen `var'_cns = `var' - `cnstr_var' //subtract out first variable of expvars from dependent variable and expectation variables
						}
					}
					ds *_cns
					loc expvars_cns `r(varlist)'
					gen `threshvar'_left_cns = (`threshvar'-`gamma_star')*left
					gen `threshvar'_right_cns = (`threshvar'-`gamma_star')*right
					gen `inflvar'_cns = `inflvar' - `cnstr_var'
					newey `inflvar'_cns `expvars_cns' `threshvar'_left_cns `threshvar'_right_cns `controls', lag(5)
					
					*collect stars
					foreach var of varlist `threshvar'_*cns `expvars_cns' {
						test `var'=0
						if r(p)<=.10 & r(p)>.05 local `var'_star "*"
						else if r(p)<=.05 & r(p)>.01 local `var'_star "**"
						else if r(p)<=.01  local `var'_star "***"
						else local `var'_star                                                                                                                 
					}					
					
					scalar nw_stderrL_2  = _se[`threshvar'_left_cns]
					scalar nw_stderrR_2  = _se[`threshvar'_right_cns]
					
					ds `expvars', not //`csntr_var'
					foreach var in `expvars_cns' {
						scalar nw_thr_`var' = _se[`var']
					}
					
					* test sum of lags = 0
					*/
					

					/*
					eststo nw_minrss //save table for final output
					
					cnsreg `inflvar' `expvars' *_left *_right `controls' ,  constraints(1-2) nocons collinear
					*Pseudo-predict 1-threshold model
					foreach var of varlist `diffur_split' `expvars' `controls' { 
						di "`var'"
						qui sum `var'
						replace `var' = r(mean)
					} 

					predict pred_`inflvar'_1thr
					keep quarter pred_`inflvar'_1thr
					
					tempfile pred_1thr //to merge "final" predicted value into dataset for graphing
					save `pred_1thr', replace
					
					
					di "`diffur_split'"
					di "`controls'"
					*/
					
				} 
				
				else {
					//di ((e(rmse)^2)*e(df_r))
					
				}
				
				// drop coefficients & re-estimate
				restore
			}
			
		local gamma_star_count `=`gamma_star_count'+1' //iteration counter	
	} //==> gamma_star loop
	
noi di gstar_2
}
