/*
	Estimates the 2v1 p-values
	
*/

/* 
	Syntax for calling this do file:
	do "`do_dir'/thresholdtests_2v1_BOOT" infl_pce_core u_rate "``var'_pcsws_ctrls'" "``var'_controls'" "`expvars'" "`sample'" "`sample_count'"
*/

args inflvar threshvar pcsws_ctrls controls expvars boot

set matsize 11000
local percentile 10
set seed 10
local tex_fmt %4.2f 


*Convert hyphenated varlist to fleshed out varlist in local
ds `expvars'
loc expvars `r(varlist)'
di "`expvars'"


/*
//check if the args passed successfully 
di "`inflvar'"
di "`threshvar'"
di "`pcsws_ctrls'"
di "`controls'"
di "`expvars'"
di "`sample'"
di "`sample_count'"
di "`boot'" 
di "`LR'"  
*/

* Simplify varnames into better labels
if "`inflvar'"=="infl_gdp" local infl_label "GDP Deflator"
if "`inflvar'"=="infl_pce_core" local infl_label "PCE Core"
if "`inflvar'"=="infl_cpi_core" local infl_label "CPI Core"
if "`inflvar'"=="infl_pce" local infl_label "PCE"
if "`inflvar'"=="infl_cpi" local infl_label "CPI"
if "`threshvar'"=="labor_share" local thresh_label "Labor Share"
if "`threshvar'"=="u_gap" local thresh_label "Unemployment Gap"
if "`threshvar'"=="u_rate" local thresh_label "Unemployment Rate"

* restrict dataset to regression sample
qui reg `inflvar' `expvars' `pcsws_ctrls' `controls'
gen tempsample = e(sample)
keep if tempsample == 1
drop tempsample

* In case we decide to include difference of urate in regression, then it needs to be averaged for graph
if "`threshvar'"!="`pcsws_ctrls'" {
	if "`inflvar'"=="infl_gdp" {
		qui ds l1_d_u_rate - l4_d_u_rate
		local diffur "`r(varlist)'"
	}	
	else {
		qui ds l0_d_u_rate - l2_d_u_rate
		local diffur "`r(varlist)'"	
	}
}

else {
	local diffur //otherwise leave blank
}	

if "`threshvar' `diffur'"!= "`pcsws_ctrls'"  {
	if "`threshvar' `diffur'"!= "`pcsws_ctrls' " { //extra space might throw Stata off (coming from empty local)
		di "`threshvar'" "`diffur'"
		di "`pcsws_ctrls'"
		di ("`threshvar'`diffur'"=="`pcsws_ctrls'")
		di "Error in pcsws_ctrls specification"
		stop //to catch mistake in code
	}
}		

* Create expecatations constraint
qui ds `expvars'
scalar n_expvars = `=wordcount("`r(varlist)'")'
loc expdesc "No lagged inflation" // for title
if n_expvars > 1 { // if more than 1 inflation expectations term
	loc expectations `=subinstr("`r(varlist)'"," ","+",.)' = 1
	loc expdesc "Lagged inflation included" // overwrite
	noi di n_expvars
	di "loc cnstr_var : word `=n_expvars' of `expvars'" //choose some variable to subtract out to imitate constraint for Newey
	loc cnstr_var : word `=n_expvars' of `expvars' //choose some variable to subtract out to imitate constraint for Newey
	noi di "`cnstr_var'"
}	

*di "`expectations'"

cap log close

* Estimate 1 threshold, then 2 thresholds; compute F-stat - 
qui { // Display only counter & results
	* initialize SSR & gamma scalars for 2 -regime grid search
	scalar rm_`inflvar'_2 = 100000
	scalar gstar_`inflvar'_2 = -666
	

	qui levelsof `threshvar' , local(u_local)
		
	
	* Setting bounds for threshold (this will be set for the rest of the file)
	qui sum `threshvar' , detail 
	scalar thresh_`percentile' = r(p`percentile') 
	scalar thresh_`=100-`percentile'' = r(p`=100-`percentile'')
	scalar thresh_spread = floor((`percentile'/100)*e(N))
	
	constraint drop _all
	// Constrain sum of inflation expectations to 1
	constraint 1 `expectations'	
	
	* 2 distinct regime model
	// Create constant
	cap drop ones
	gen ones = 1 
	
	
	local gamma_star_count 1
	foreach gamma_star in `u_local' {

			if (`gamma_star' >= thresh_`percentile') & (`gamma_star' <= thresh_`=100-`percentile'')  { //OB bounds for threshold values (adjustable by percentile)

				preserve

				
				gen left = .
				replace left = 1 if `threshvar'<=`gamma_star'
				replace left = 0 if mi(left)

				gen right = .
				replace right = 1 if !(`threshvar'<=`gamma_star')
				replace right = 0 if mi(right)
				
				constraint drop 2
				constraint 2 `gamma_star'*(`threshvar'_left-`threshvar'_right) + (ones_left-ones_right) = 0
				
				* create constant & urate (contemp & lagged diff) for each state
				gen ones_left = ones*left
				gen ones_right = ones*right
				
				local diffur_split
				
				foreach vv in `threshvar' `diffur' {
					gen `vv'_left = `vv'*left
					gen `vv'_right = `vv'*right
					if "`vv'"!="`threshvar'" { //for taking the mean 
						local diffur_split `diffur_split' `vv'_left `vv'_right	
					}		
				}
				
				// Estimate

				cnsreg `inflvar' `expvars' *_left *_right `controls' ,  constraints(1-2) nocons collinear

				// Collect slope estimates
				scalar bL_2 = _b[`threshvar'_left]
				scalar SE_L_2 = _se[`threshvar'_left]
				scalar bR_2 = _b[`threshvar'_right]
				scalar SE_R_2 = _se[`threshvar'_right]
				
			
				// Minimization for cnsreg
				if ((e(rmse)^2)*e(df_r))<rm_`inflvar'_2 { //cnsreg doesn't report e(rss), so it must be calculated as (root mean squared error)^2 * (residual degrees of freedom)
					//eststo minrss //save table for final output
					loc n_obs_2 = `=e(N)'
					scalar rm_`inflvar'_2 =((e(rmse)^2)*e(df_r))
					scalar gstar_`inflvar'_2 = `gamma_star'
					scalar betaL_2 = bL_2
					scalar stderrL_2 = SE_L_2
					scalar betaR_2 = bR_2
					scalar stderrR_2 = SE_R_2	
	
					
					*F-statistic for sum of lags
					/*test l1_`inflvar' + l2_`inflvar' + l3_`inflvar' + l4_`inflvar' + l5_`inflvar' = 0
					scalar lagsum_p = r(p)
					lincom l1_`inflvar' + l2_`inflvar' + l3_`inflvar' + l4_`inflvar' + l5_`inflvar'
					scalar lagsum_coef = r(estimate)
					
					local F_`inflvar'_est  `=`obs_n_`inflvar''*(`F_lin_`inflvar'' - rm_`inflvar'_2)/rm_`inflvar'_2'
					*/
					//noi di "2 Distinct Regimes, `inflvar' Sample: `sample', `gamma_star_count'"
					
					*Newey-West with Regression Kink Model
					/*
					cap drop *_cns
					ds `expvars'
					foreach var in `r(varlist)' {
						if `var'!=`cnstr_var' {
							gen `var'_cns = `var' - `cnstr_var' //subtract out first variable of expvars from dependent variable and expectation variables
						}
					}
					ds *_cns
					loc expvars_cns `r(varlist)'
					gen `threshvar'_left_cns = (`threshvar'-`gamma_star')*left
					gen `threshvar'_right_cns = (`threshvar'-`gamma_star')*right
					gen `inflvar'_cns = `inflvar' - `cnstr_var'
					newey `inflvar'_cns `expvars_cns' `threshvar'_left_cns `threshvar'_right_cns `controls', lag(5)
					eststo nw_minrss //save table for final output
					*/
					cnsreg `inflvar' `expvars' *_left *_right `controls' ,  constraints(1-2) nocons collinear
					*Pseudo-predict 1-threshold model
					foreach var of varlist `diffur_split' `expvars' `controls' { 
						di "`var'"
						qui sum `var'
						replace `var' = r(mean)
					} 
					
					predict pred_`inflvar'_1thr
					keep quarter pred_`inflvar'_1thr
					
					tempfile pred_1thr //to merge "final" predicted value into dataset for graphing
					save `pred_1thr', replace
					
					
					di "`diffur_split'"
					di "`controls'"
					

					
				} 
				
				else {
					di ((e(rmse)^2)*e(df_r))
					
				}
				
				// drop coefficients & re-estimate
				restore
			}
			
		local gamma_star_count `=`gamma_star_count'+1' //iteration counter
	} //==> gamma_star loop
		**** 2 regime estimation done -  Using SSR to compute F-stats for 2 vs 3 regimes hyp test

	* 3 distinct regime model
	* initialize SSR & gamma scalars for 2 -regime grid search
	scalar rm_`inflvar' = 100000
	scalar gl_`inflvar' = -666
	scalar gu_`inflvar' = -666
	
	// Create constant
	cap drop ones
	gen ones = 1 
	
	local gamma_l_count 1
	foreach gamma_l in `u_local' {

		local gamma_u_count 1
		foreach gamma_u in `u_local' {
			count if `threshvar'>=`gamma_l' & `threshvar'<=`gamma_u'
			if r(N)>=thresh_spread & (`gamma_l' >= thresh_`percentile') & (`gamma_u' <= thresh_`=100-`percentile'')  { //OB bounds for threshold values (adjustable by percentile)
			*if (`gamma_l'>=u_25)  & (`gamma_u'<=u_75) {
			
				constraint drop 2 3
				di "Counter ....."
				di `gamma_l_count'
				di `gamma_u_count'
				
				preserve
				
				
				gen inside = .
				replace inside = 1 if `threshvar'>=`gamma_l' & `threshvar'<=`gamma_u'
				replace inside = 0 if mi(inside)

				gen above = .
				replace above = 1 if `threshvar'>`gamma_u' 
				replace above = 0 if mi(above)
				
				gen below = .
				replace below = 1 if `threshvar'<`gamma_l'
				replace below = 0 if mi(below)
					
				* create constant & urate (contemp & lagged diff) for each state
				gen ones_inside = ones*inside
				gen ones_above = ones*above
				gen ones_below = ones*below
				
				local diffur_split
				
				foreach vv in `threshvar' `diffur' {
					gen `vv'_inside = `vv'*inside
					gen `vv'_above = `vv'*above
					gen `vv'_below = `vv'*below
					if "`vv'"!="`threshvar'" { //for taking the mean 
						local diffur_split `diffur_split' `vv'_inside `vv'_below `vv'_above	
					}		
				}
				
				* Define continuity constraints
				constraint 2 `gamma_l'*(`threshvar'_below-`threshvar'_inside) + (ones_below-ones_inside) = 0
				constraint 3 `gamma_u'*(`threshvar'_inside-`threshvar'_above) + (ones_inside-ones_above) = 0
				
				cnsreg `inflvar' `expvars' *_inside *_above *_below `controls' ,  constraints(1-3) nocons collinear

				// Collect slope estimates
				scalar bI = _b[`threshvar'_inside]
				scalar SE_I = _se[`threshvar'_inside]
				scalar bA = _b[`threshvar'_above]
				scalar SE_A = _se[`threshvar'_above]
				scalar bB = _b[`threshvar'_below]
				scalar SE_B = _se[`threshvar'_below]
				
				// Minimization for cnsreg
				if ((e(rmse)^2)*e(df_r))<rm_`inflvar' { //cnsreg doesn't report e(rss), so it must be calculated as (root mean squared error)^2 * (residual degrees of freedom)

					
					scalar rm_`inflvar' =((e(rmse)^2)*e(df_r))
					scalar gl_`inflvar' = `gamma_l'
					scalar gu_`inflvar' = `gamma_u'
					scalar betaI = bI
					scalar stderrI = SE_I
					scalar betaA = bA
					scalar stderrA = SE_A
					scalar betaB = bB
					scalar stderrB = SE_B		
					
					test `threshvar'_inside `threshvar'_above `threshvar'_below
					scalar pIAB = r(p)
					test `threshvar'_inside `threshvar'_above 
					scalar pIA = r(p)
					test `threshvar'_inside `threshvar'_below
					scalar pIB  = r(p)
					test `threshvar'_above `threshvar'_below
					scalar pAB  = r(p)
					
					local counter_`inflvar'  "`gamma_l_count'_`gamma_u_count'"
					//noi di "3 Distinct Regimes - `inflvar' Sample: `sample', `counter_`inflvar''"
					// Compute F-stat
					local F_`inflvar'_est  `=`n_obs_2'*(rm_`inflvar'_2 - rm_`inflvar')/rm_`inflvar''
					
					di "`diffur_split'"
					di "`controls'"
				} 
				
				else {
					di ((e(rmse)^2)*e(df_r))
					
				}
				
				// drop coefficients & re-estimate
				restore
			}
			local gamma_u_count `=`gamma_u_count'+1' //iteration counter	
		} //==> gamma_u loop
		local gamma_l_count `=`gamma_l_count'+1' //iteration counter	
	} //==> gamma_l loop

} // end output suppression


* Bootstrap to CIs for params
if "`boot'" == "Y" {
qui { // Display only counter & results
loc gstar = gstar_`inflvar'_2 // estimated threshold pt
loc beta1_star = betaL_2 // estimated slope
loc beta2_star = betaR_2 // estimated slope
loc F_est = `F_`inflvar'_est' // Estimated F stat

* Construct resids taking gammastar as given & linear resids for p value
		cap drop yhat linfits resid linresid ones* left right
		constraint drop _all	

		// Constrain sum of inflation expectations to 1
		constraint 1 `expectations'

		if n_expvars > 1 {
			cnsreg `inflvar' `expvars' `pcsws_ctrls' `controls' , constraints(1)
		}
		else {
			reg `inflvar' `expvars' `pcsws_ctrls' `controls' // reg still saves rmse & df_r in e()
		}
		predict linfits, xb
		predict linresid, res					
		
		cap drop *left* *right*
		gen left = .
		replace left = 1 if `threshvar'<=`gstar'
		replace left = 0 if mi(left)

		gen right = .
		replace right = 1 if !(`threshvar'<=`gstar')
		replace right = 0 if mi(right)
		
		constraint 2 `gstar'*(`threshvar'_left-`threshvar'_right) + (ones_left-ones_right) = 0
		
		* create constant & slack var for each state
		gen ones_left = ones*left
		gen ones_right = ones*right
		
		foreach vv in `threshvar' {
			gen `vv'_left = `vv'*left
			gen `vv'_right = `vv'*right		
		}
		
		// Compute fitted values and residuals, taking threshold as given
		cnsreg `inflvar' `expvars' *_left *_right `controls' ,  constraints(1-2) nocons collinear		
		local obs_n `=e(N)'
		
		predict yhat, xb
		predict resid, res			
		
		// preallocate to store bootstrapped values
		loc n_sim = 200
		matrix Fboot = J(`n_sim',1,.)
		matrix Fboot_lin = J(`n_sim',1,.)
		matrix gammaBoot = J(`n_sim',1,.)
		matrix beta1Boot= J(`n_sim',1,.)
		matrix beta2Boot= J(`n_sim',1,.)
		
		forval s = 1/`n_sim' {
			noi di "3 Distinct Regimes - `inflvar',`threshvar'; Iteration: `s'"
			sort quarter
			//gen n_resid=_n 
			//order n_resid quarter
			
			cap drop *resid_boot `inflvar'_*boot
			
			*Transform residuals in wild bootstrap			
			gen resid_boot = resid*rnormal(0,1)
			gen linresid_boot = linresid*rnormal(0,1)

			
	
		// create resampled y for threshold and linear models
		gen `inflvar'_linboot = linfits+linresid_boot
		gen `inflvar'_boot = yhat+resid_boot
		


		* initialize SSR & gamma scalars
		scalar rm_`inflvar'_2_boot = 100000

				
		qui sum `threshvar' , detail 
		qui levelsof `threshvar' , local(u_local)
		
		* Grid search using resampled y
		// Create constant
		cap drop ones
		gen ones = 1 
		
		* Estimate 2 regime model with resampled H0 residuals
		local gamma_star_count 1
		foreach gamma_star in `u_local' { //***********************************

				if (`gamma_star' >= thresh_`percentile') & (`gamma_star' <= thresh_`=100-`percentile'')  { //OB bounds for threshold values (adjustable by percentile)

					di "Counter ....."
					di `gamma_star_count'
					
					preserve

					cap drop *left* *right*
					gen left = .
					replace left = 1 if `threshvar'<=`gamma_star'
					replace left = 0 if mi(left)

					gen right = .
					replace right = 1 if !(`threshvar'<=`gamma_star')
					replace right = 0 if mi(right)
					
					constraint 2 `gamma_star'*(`threshvar'_left-`threshvar'_right) + (ones_left-ones_right) = 0
					
					* create constant & urate (contemp & lagged diff) for each state
					gen ones_left = ones*left
					gen ones_right = ones*right
					
					local diffur_split
					
					foreach vv in `threshvar' `diffur' {
						gen `vv'_left = `vv'*left
						gen `vv'_right = `vv'*right
						if "`vv'"!="`threshvar'" { //for taking the mean 
							local diffur_split `diffur_split' `vv'_left `vv'_right	
						}		
					}
					
					// Estimate

					cnsreg `inflvar'_boot `expvars' *_left *_right `controls' ,  constraints(1-2) nocons collinear

					// Minimization for cnsreg
					if ((e(rmse)^2)*e(df_r))<rm_`inflvar'_2_boot { //cnsreg doesn't report e(rss), so it must be calculated as (root mean squared error)^2 * (residual degrees of freedom)
						local obs_n_`inflvar'_boot `=e(N)'
						scalar rm_`inflvar'_2_boot =((e(rmse)^2)*e(df_r))
						local F_H0_`inflvar'_boot =((e(rmse)^2)*e(df_r))
						scalar gstar_`inflvar'_2 = `gamma_star'
						scalar betaL_2 = bL_2
						scalar stderrL_2 = SE_L_2
						scalar betaR_2 = bR_2
						scalar stderrR_2 = SE_R_2		
						
						local counter_`inflvar'  "`gamma_star_count'"
						//noi di "2 Distinct Regimes, `inflvar' Sample: `sample', `counter_`inflvar''"
						
					} 
					
					else {
						di ((e(rmse)^2)*e(df_r))
					}

					// drop coefficients & re-estimate
					restore
				}
				
			local gamma_star_count `=`gamma_star_count'+1' //iteration counter	
		} //==> gamma_star loop
				////////////////////////************************

		cap drop ones *left *right
		
		* Estimate 3 regime model with resample H0 residuals
		
		* initialize SSR & gamma scalars
		scalar rm_`inflvar'_boot = 100000
		scalar gl_`inflvar'_boot = -666
		scalar gu_`inflvar'_boot = -666

		qui sum `threshvar' , detail 
		
		qui levelsof `threshvar' , local(u_local)
		
		// Create constant
		cap drop ones // generated constant & piecewise variables
		gen ones = 1 
		
		local gamma_l_count 1
		foreach gamma_l in `u_local' {
		
			local gamma_u_count 1
			foreach gamma_u in `u_local' {
			count if `threshvar'>=`gamma_l' & `threshvar'<=`gamma_u'
			if r(N)>=thresh_spread & (`gamma_l' >= thresh_`percentile') & (`gamma_u' <= thresh_`=100-`percentile'') { //OB bounds for threshold values (temporary)
				
				constraint drop 2 3
				di "Counter ....."
				di `gamma_l_count'
				di `gamma_u_count'
				
				preserve
				
				cap drop *inside* *above* *below*
				gen inside = .
				replace inside = 1 if `threshvar'>=`gamma_l' & `threshvar'<=`gamma_u'
				replace inside = 0 if mi(inside)

				gen above = .
				replace above = 1 if `threshvar'>`gamma_u' 
				replace above = 0 if mi(above)
				
				gen below = .
				replace below = 1 if `threshvar'<`gamma_l'
				replace below = 0 if mi(below)
					
				* create constant & urate (contemp & lagged diff) for each state
				gen ones_inside = ones*inside
				gen ones_above = ones*above
				gen ones_below = ones*below
				
				foreach vv in `pcsws_ctrls' {
					gen `vv'_inside = `vv'*inside
					gen `vv'_above = `vv'*above
					gen `vv'_below = `vv'*below
				}	
					
				* Define constraint
				constraint 2 `gamma_l'*(`threshvar'_below-`threshvar'_inside) + (ones_below-ones_inside) = 0
				constraint 3 `gamma_u'*(`threshvar'_inside-`threshvar'_above) + (ones_inside-ones_above) = 0
				
				cnsreg `inflvar'_boot `expvars' *_inside *_above *_below `controls' ,  constraints(1-3) nocons collinear
			
				
				// Collect slope estimates

				scalar bI = _b[`threshvar'_inside]
				scalar SE_I = _se[`threshvar'_inside]
				scalar bA = _b[`threshvar'_above]
				scalar SE_A = _se[`threshvar'_above]
				scalar bB = _b[`threshvar'_below]
				scalar SE_B = _se[`threshvar'_below]
				
			
				// Minimization for cnsreg
				if ((e(rmse)^2)*e(df_r))<rm_`inflvar'_boot { //cnsreg doesn't report e(rss), so it must be calculated as (root mean squared error)^2 * (residual degrees of freedom)
					scalar rm_`inflvar'_boot =((e(rmse)^2)*e(df_r))
					scalar gl_`inflvar'_boot = `gamma_l'
					scalar gu_`inflvar'_boot = `gamma_u'					
					local counter_`inflvar'  "`gamma_l_count'_`gamma_u_count'"
					//noi di "`counter_`inflvar''"
					local F_boot  `=`obs_n_`inflvar'_boot'*(`F_H0_`inflvar'_boot' - rm_`inflvar'_boot)/rm_`inflvar'_boot'
					
				}
				else {
					di ((e(rmse)^2)*e(df_r))
					
				}
				
				// drop coefficients & re-estimate
				restore
				}
				local gamma_u_count `=`gamma_u_count'+1' //iteration counter	
			} //==> gamma_u loop
			local gamma_l_count `=`gamma_l_count'+1' //iteration counter	
		} //==> gamma_l loop
		* Save the F-stats, slopes and gammas from bootstrapping 
		matrix Fboot[`s',1] = `F_boot'
		//matrix gammaBoot[`s',1] = gstar_`inflvar'_boot
		//matrix beta1Boot[`s',1] =bL_boot
		//matrix beta2Boot[`s',1] =bR_boot
		
} // over bootstrap draws


*Generate bootstrapped CIs
preserve
foreach mm in Fboot {
	svmat `mm', names(`mm')
}

// Create p-values for existence of threshold
count if Fboot1 >= `F_est' & !mi(Fboot1)
scalar pval = `r(N)'/`n_sim'
histogram Fboot1

restore

} // end output suppression
}
