/*
	This files estimates the threshold for the data pre-1990 and saves result to esttab, for 
	generation in latex later. (for split sample)
*/

/* 
	Syntax for calling this do file:
	do "`do_dir'/estimationTables" infl_pce_core u_rate "``var'_pcsws_ctrls'" "``var'_controls'" "`expvars'"
*/

args inflvar threshvar pcsws_ctrls controls expvars boot
set matsize 11000

local percentile 10
set seed 10
local tex_fmt %4.2f 


*Convert hyphenated varlist to fleshed out varlist in local
ds `expvars'
loc expvars `r(varlist)'
di "`expvars'"

*Create Labels for Table 
loc lab_exp_infl_MI "Michigan Expectations"
loc lab_exp_infl_SPF_gdp "SPF Expectations"
loc lab_l1_`inflvar' "Lag 1"
loc lab_l2_`inflvar' "Lag 2"
loc lab_l3_`inflvar' "Lag 3"
loc lab_l4_`inflvar' "Lag 4"
loc lab_l5_`inflvar' "Lag 5"

*Period 1 (Oil Shock Inflation)
loc period_1_start `=q(1976q2)'
loc period_1_end `=q(1980q1)'

*Period 2 (Volcker)
loc period_2_start `=q(1980q2)'
loc period_2_end `=q(1985q4)'

*Period 3 (Great Recession)
loc period_3_start `=q(2009q1)'
loc period_3_end `=q(2013q4)'

count if quarter>=`period_1_start' & quarter<=`period_1_end'
loc obs_n1_`inflvar' `=r(N)' //observations in period 1
count if quarter>=`period_2_start' & quarter<=`period_2_end'
loc obs_n2_`inflvar' `=r(N)' //observations in period 2
count if quarter>=`period_3_start' & quarter<=`period_3_end'
loc obs_n3_`inflvar' `=r(N)' //observations in period 3
di `obs_n3_`inflvar''


* restrict dataset to regression sample
qui reg `inflvar' `expvars' `pcsws_ctrls' `controls'
gen tempsample = e(sample)
keep if tempsample == 1
drop tempsample

* In case we decide to include difference of urate in regression, then it needs to be averaged for graph
if "`threshvar'"!="`pcsws_ctrls'" {
	qui ds l0_d_u_rate - l2_d_u_rate
	local diffur "`r(varlist)'"	
}

else {
	local diffur //otherwise leave blank
}	

if "`threshvar' `diffur'"!= "`pcsws_ctrls'"  {
	if "`threshvar' `diffur'"!= "`pcsws_ctrls' " { //extra space might throw Stata off (coming from empty local)
		di "`threshvar'" "`diffur'"
		di "`pcsws_ctrls'"
		di ("`threshvar'`diffur'"=="`pcsws_ctrls'")
		di "Error in pcsws_ctrls specification"
		stop //to catch mistake in code
	}
}		

* Create expecatations constraint
qui ds `expvars'

scalar n_expvars = `=wordcount("`r(varlist)'")'
loc expdesc "No lagged inflation" // for title
if n_expvars > 1 { // if more than 1 inflation expectations term
	loc expectations `=subinstr("`r(varlist)'"," ","+",.)' = 1
	loc expdesc "Lagged inflation included" // overwrite
	noi di n_expvars
	di "loc cnstr_var : word `=n_expvars' of `expvars'" //choose some variable to subtract out to imitate constraint for Newey
	loc cnstr_var : word `=n_expvars' of `expvars' //choose some variable to subtract out to imitate constraint for Newey
	noi di "`cnstr_var'"
}	


******Cut the sample**********
keep if quarter<q(1990q1)
******************************

cap log close

* Estimate 1 threshold, then 2 thresholds; compute F-stat - 
qui { // Display only counter & results
	* initialize SSR & gamma scalars for 2 -regime grid search
	scalar rm_`inflvar'_2 = 100000
	scalar gstar_`inflvar'_2 = -666
	
	constraint drop _all
	// Constrain sum of inflation expectations to 1
	constraint 1 `expectations'
	
	if n_expvars > 1 {
		cnsreg `inflvar' `expvars' `pcsws_ctrls' `controls' , constraints(1)
	}
	else {
		reg `inflvar' `expvars' `pcsws_ctrls' `controls' // reg still saves rmse & df_r in e()
	}
	
	local F_lin_`inflvar' `=(e(rmse)^2)*e(df_r)' // cnsreg doesn't save e(rss)
	
	
	**Collect subsample RSS for linear model
	predict lin_resid, res //collect residuals in variable
	replace lin_resid = lin_resid^2
	//collect RSS in of 3 different subsamples in 3 different variables
	egen lin_resid1 = sum(lin_resid) if quarter>=`period_1_start' & quarter<=`period_1_end'
	egen lin_resid2 = sum(lin_resid) if quarter>=`period_2_start' & quarter<=`period_2_end' 
	egen lin_resid3 = sum(lin_resid) if quarter>=`period_3_start' & quarter<=`period_3_end'
	forval i=1/3 { //RSS = max = mean = min for each variable ==> place number into a local
		sum lin_resid`i'
		loc F`i'_lin_`inflvar' `=r(mean)'
	}	
	drop lin_resid
	
	local obs_n_`inflvar' `=e(N)'
	scalar b_lin = _b[`threshvar']
	scalar SE_lin = _se[`threshvar']
	foreach var in `expvars' {
		scalar b_lin_`var' = _b[`var']
		scalar SE_lin_`var' = _se[`var']
	}

	lincom l1_`inflvar' + l2_`inflvar' + l3_`inflvar' + l4_`inflvar' + l5_`inflvar'
	scalar lin_lagsum_coef = r(estimate)
	scalar lin_lagsum_se = r(se)
	test l1_`inflvar' + l2_`inflvar' + l3_`inflvar' + l4_`inflvar' + l5_`inflvar' = 0
	//scalar lin_lagsum_p = r(p)

	if r(p)<=.10 & r(p)>.05 local lin_lagsum_coef_star "*"
	else if r(p)<=.05 & r(p)>.01 local lin_lagsum_coef_star "**"
	else if r(p)<=.01  local lin_lagsum_coef_star "***"
	else local lin_lagsum_coef_star 
	
	*Newey West
	
	preserve
	cap drop *_cns
	ds `expvars'
	foreach var in `r(varlist)' {
		if `var'!=`cnstr_var' {
			gen `var'_cns = `var' - `cnstr_var' //subtract out first variable of expvars from dependent variable and expectation variables
		}
	}
	ds *_cns
	loc expvars_cns `r(varlist)'
	gen `inflvar'_cns = `inflvar' - `cnstr_var'
	newey `inflvar'_cns `expvars_cns' `pcsws_ctrls' `controls', lag(5) force
	
	*Assign stars to coefficients
	foreach var in `threshvar' `expvars_cns' {
		test `var'=0
		if r(p)<=.10 & r(p)>.05 local b_lin_`var'_star "*"
		else if r(p)<=.05 & r(p)>.01 local b_lin_`var'_star "**"
		else if r(p)<=.01  local b_lin_`var'_star "***"
		else local b_lin_`var'_star 
	}

	eststo lin_`inflvar'_`threshvar'

	scalar nw_SE_lin = _se[`threshvar']
	foreach var in `expvars_cns' {
		scalar nw_lin_`var' = _se[`var']
	}
	restore
	
	qui levelsof `threshvar' , local(u_local)
		
	
	* Setting bounds for threshold (this will be set for the rest of the file)
	qui sum `threshvar' , detail 
	scalar thresh_`percentile' = r(p`percentile') 
	scalar thresh_`=100-`percentile'' = r(p`=100-`percentile'')
	scalar thresh_spread = floor((`percentile'/100)*e(N))
	
	constraint drop _all
	// Constrain sum of inflation expectations to 1
	constraint 1 `expectations'	
	
	* 2 distinct regime model
	// Create constant
	cap drop ones
	gen ones = 1 
	
	
	local gamma_star_count 1
	foreach gamma_star in `u_local' {

			if (`gamma_star' >= thresh_`percentile') & (`gamma_star' <= thresh_`=100-`percentile'')  { //OB bounds for threshold values (adjustable by percentile)

				di "Counter ....."
				di `gamma_star_count'
				
				preserve

				
				gen left = .
				replace left = 1 if `threshvar'<=`gamma_star'
				replace left = 0 if mi(left)

				gen right = .
				replace right = 1 if !(`threshvar'<=`gamma_star')
				replace right = 0 if mi(right)
				
				constraint drop 2
				constraint 2 `gamma_star'*(`threshvar'_left-`threshvar'_right) + (ones_left-ones_right) = 0
				
				* create constant & urate (contemp & lagged diff) for each state
				gen ones_left = ones*left
				gen ones_right = ones*right
				
				local diffur_split
				
				foreach vv in `threshvar' `diffur' {
					gen `vv'_left = `vv'*left
					gen `vv'_right = `vv'*right
					if "`vv'"!="`threshvar'" { //for taking the mean 
						local diffur_split `diffur_split' `vv'_left `vv'_right	
					}		
				}
				
				// Estimate

				cnsreg `inflvar' `expvars' *_left *_right `controls' ,  constraints(1-2) nocons collinear
				
				
				// Collect slope estimates
				scalar bL_2 = _b[`threshvar'_left]
				scalar SE_L_2 = _se[`threshvar'_left]
				scalar bR_2 = _b[`threshvar'_right]
				scalar SE_R_2 = _se[`threshvar'_right]
		
			
				// Minimization for cnsreg
				if ((e(rmse)^2)*e(df_r))<rm_`inflvar'_2 { //cnsreg doesn't report e(rss), so it must be calculated as (root mean squared error)^2 * (residual degrees of freedom)
					eststo minrss //save table for final output
					loc n_obs_2 = `=e(N)'
					scalar rm_`inflvar'_2 =((e(rmse)^2)*e(df_r))
					scalar gstar_`inflvar'_2 = `gamma_star'
					scalar betaL_2 = bL_2
					*local betaL_2_star "``threshvar'_left_star'"
					scalar stderrL_2 = SE_L_2
					scalar betaR_2 = bR_2
					*local betaR_2_star "``threshvar'_right_star'"
					scalar stderrR_2 = SE_R_2	
					foreach var in `expvars' {
						scalar b_thr_`var' = _b[`var']
						*local b_thr_`var'_star "``var'_star'"
						scalar SE_thr_`var' = _se[`var']
					}	
					
					* R-sq
					summ `inflvar', d
					loc Rsq `= 1 - (rm_`inflvar'_2 /(r(Var)*`n_obs_2'))'

					*F-statistic for sum of lags
					test l1_`inflvar' + l2_`inflvar' + l3_`inflvar' + l4_`inflvar' + l5_`inflvar' = 0
					if r(p)<=.10 & r(p)>.05 local lagsum_coef_star "*"
					else if r(p)<=.05 & r(p)>.01 local lagsum_coef_star "**"
					else if r(p)<=.01  local lagsum_coef_star "***"
					else local lagsum_coef_star 
					lincom l1_`inflvar' + l2_`inflvar' + l3_`inflvar' + l4_`inflvar' + l5_`inflvar'
					scalar lagsum_coef = r(estimate)
					scalar lagsum_se = r(se)
					local F_`inflvar'_est  `=`obs_n_`inflvar''*(`F_lin_`inflvar'' - rm_`inflvar'_2)/rm_`inflvar'_2'
					
					*F-statistic for subsample periods
					predict thresh_resid, res //collect residuals in variable
					replace thresh_resid = thresh_resid^2
					//collect RSS in of 3 different subsamples in 3 different variables
					egen thresh_resid1 = sum(thresh_resid) if quarter>=`period_1_start' & quarter<=`period_1_end'
					egen thresh_resid2 = sum(thresh_resid) if quarter>=`period_2_start' & quarter<=`period_2_end' 
					egen thresh_resid3 = sum(thresh_resid) if quarter>=`period_3_start' & quarter<=`period_3_end'
					forval i=1/3 { //RSS = max = mean = min for each variable ==> place number into a local
						sum thresh_resid`i'
						loc rm_`inflvar'_sub`i' `=r(mean)'
					}
					local F1_`inflvar'_est  `=`obs_n1_`inflvar''*(`F1_lin_`inflvar'' - `rm_`inflvar'_sub1')/`rm_`inflvar'_sub1''
					local F2_`inflvar'_est  `=`obs_n2_`inflvar''*(`F2_lin_`inflvar'' - `rm_`inflvar'_sub2')/`rm_`inflvar'_sub2''
					local F3_`inflvar'_est  `=`obs_n3_`inflvar''*(`F3_lin_`inflvar'' - `rm_`inflvar'_sub3')/`rm_`inflvar'_sub3''
					
					drop thresh_resid*
					noi di "2 Distinct Regimes, `inflvar' `threshvar', `gamma_star_count'"
					
					*Newey-West with Regression Kink Model
					
					cap drop *_cns
					ds `expvars'
					foreach var in `r(varlist)' {
						if `var'!=`cnstr_var' {
							gen `var'_cns = `var' - `cnstr_var' //subtract out first variable of expvars from dependent variable and expectation variables
						}
					}
					ds *_cns
					loc expvars_cns `r(varlist)'
					gen `threshvar'_left_cns = (`threshvar'-`gamma_star')*left
					gen `threshvar'_right_cns = (`threshvar'-`gamma_star')*right
					gen `inflvar'_cns = `inflvar' - `cnstr_var'
					newey `inflvar'_cns `expvars_cns' `threshvar'_left_cns `threshvar'_right_cns `controls', lag(5) force
					
					*collect stars
					foreach var of varlist `threshvar'_*cns `expvars_cns' {
						test `var'=0
						if r(p)<=.10 & r(p)>.05 local `var'_star "*"
						else if r(p)<=.05 & r(p)>.01 local `var'_star "**"
						else if r(p)<=.01  local `var'_star "***"
						else local `var'_star 
					}					
					
					scalar nw_stderrL_2  = _se[`threshvar'_left_cns]
					scalar nw_stderrR_2  = _se[`threshvar'_right_cns]
					
					ds `expvars', not //`csntr_var'
					foreach var in `expvars_cns' {
						scalar nw_thr_`var' = _se[`var']
					}
					
					* test sum of lags = 0
					if regexm("`expvars'", "exp_infl_SPF_gdp") & regexm("`expvars'", "exp_infl_MI") {
						lincom exp_infl_SPF_gdp_cns + exp_infl_MI_cns - 1
						scalar lagsumNW_coef = -1*r(estimate)
						scalar lagsumNW_se = r(se)	
						
						scalar z = abs(lagsumNW_coef/lagsumNW_se)

						scalar p = exp((0.743 - (z+0.862)^2)/2.404)
						
						if p < 0.01 {
							loc lagsumNW_coef_star "***"
						}
						else if p >= 0.01 & p < 0.05 {
							loc lagsumNW_coef_star "**"					
						}
						else if p >= 0.05 & p < 0.1 {
							loc lagsumNW_coef_star "*"					
						}						
						else {
							loc lagsumNW_coef_star ""
						}
						
					}
					else if regexm("`expvars'", "exp_infl_SPF_gdp") & !regexm("`expvars'", "exp_infl_MI") {
						lincom exp_infl_SPF_gdp_cns - 1
						scalar lagsumNW_coef = -1*r(estimate)
						scalar lagsumNW_se = r(se)
						
						scalar z = abs(lagsumNW_coef/lagsumNW_se)
						scalar p = exp((0.743 - (z+0.862)^2)/2.404)
						
						if p < 0.01 {
							loc lagsumNW_coef_star "***"
						}
						else if p >= 0.01 & p < 0.05 {
							loc lagsumNW_coef_star "**"					
						}
						else if p >= 0.05 & p < 0.1 {
							loc lagsumNW_coef_star "*"					
						}				
						else {
							loc lagsumNW_coef_star ""
						}						
						
					}
					else {
						n di "Is this model supposed to be lags only?"
						nr
					}
					
					eststo t_`inflvar'_`threshvar' //save table for final output

					

					cnsreg `inflvar' `expvars' *_left *_right `controls' ,  constraints(1-2) nocons collinear
					*Pseudo-predict 1-threshold model
					foreach var of varlist `diffur_split' `expvars' `controls' { 
						di "`var'"
						qui sum `var'
						replace `var' = r(mean)
					} 

					predict pred_`inflvar'_1thr
					keep quarter pred_`inflvar'_1thr
					
					tempfile pred_1thr //to merge "final" predicted value into dataset for graphing
					save `pred_1thr', replace
					
					
					di "`diffur_split'"
					di "`controls'"
					
				} 
				
				else {
					di ((e(rmse)^2)*e(df_r))
					
				}
				
				// drop coefficients & re-estimate
				restore
			}
			
		local gamma_star_count `=`gamma_star_count'+1' //iteration counter	
	} //==> gamma_star loop

* end of estimation

// save some scalars for output
scalar Rsq = `=`Rsq''
scalar nobs = `=`n_obs_2''
} // end output suppression

* Bootstrap to CIs for params
if "`boot'" == "Y" {
	
qui { // Display only counter & results

loc gstar = gstar_`inflvar'_2 // estimated threshold pt
loc beta1_star = betaL_2 // estimated slope
loc beta2_star = betaR_2 // estimated slope
loc F_est = `F_`inflvar'_est' // Estimated F stat
loc F1_est = `F1_`inflvar'_est' // Estimated F stat (subsample: Period 1)
loc F2_est = `F2_`inflvar'_est' // Estimated F stat (subsample: Period 1)
loc F3_est = `F3_`inflvar'_est' // Estimated F stat (subsample: Period 1)

* Construct resids taking gammastar as given & linear resids for p value
		cap drop yhat linfits resid linresid ones* left right
		constraint drop 2	

		// Constrain sum of inflation expectations to 1
		constraint 1 `expectations'

		if n_expvars > 1 {
			cnsreg `inflvar' `expvars' `pcsws_ctrls' `controls' , constraints(1)
		}
		else {
			reg `inflvar' `expvars' `pcsws_ctrls' `controls' // reg still saves rmse & df_r in e()
		}
		predict linfits, xb
		predict linresid, res					
		
		gen left = .
		replace left = 1 if `threshvar'<=`gstar'
		replace left = 0 if mi(left)

		gen right = .
		replace right = 1 if !(`threshvar'<=`gstar')
		replace right = 0 if mi(right)
		
		constraint 2 `gstar'*(`threshvar'_left-`threshvar'_right) + (ones_left-ones_right) = 0
		
		* create constant & slack var for each state
		gen ones_left = ones*left
		gen ones_right = ones*right
		
		foreach vv in `threshvar' {
			gen `vv'_left = `vv'*left
			gen `vv'_right = `vv'*right		
		}
		
		// Compute fitted values and residuals, taking threshold as given
		cnsreg `inflvar' `expvars' *_left *_right `controls' ,  constraints(1-2) nocons collinear		
		local obs_n `=e(N)'
		
		predict yhat, xb
		predict resid, res			
		
		// preallocate to store bootstrapped values
		loc n_sim = 1000
		matrix Fboot = J(`n_sim',1,.)
		matrix Fboot_lin = J(`n_sim',1,.)
		matrix F1boot_lin =  J(`n_sim',1,.) 
		matrix F2boot_lin =  J(`n_sim',1,.)
		matrix F3boot_lin =  J(`n_sim',1,.) 
		matrix gammaBoot = J(`n_sim',1,.)
		matrix beta1Boot= J(`n_sim',1,.)
		matrix beta2Boot= J(`n_sim',1,.)
		
		forval s = 1/`n_sim' {
			noi di "`inflvar', `threshvar', Seed `s'"
			sort quarter
			//gen n_resid=_n 
			//order n_resid quarter
			
			cap drop *resid_boot `inflvar'_*boot
			
			*Transform residuals in wild bootstrap			
			gen resid_boot = resid*rnormal(0,1)
			gen linresid_boot = linresid*rnormal(0,1)

			
	
		// create resampled y for threshold and linear models
		gen `inflvar'_linboot = linfits+linresid_boot
		gen `inflvar'_boot = yhat+resid_boot
		
		* SSR from linear model w/ resampled y
		if n_expvars > 1 {
			cnsreg `inflvar'_linboot `expvars' `pcsws_ctrls' `controls' , constraints(1)
		}
		else {
			reg `inflvar'_linboot `expvars' `pcsws_ctrls' `controls'
		}		
		local F_lin_`inflvar'_boot `=(e(rmse)^2)*e(df_r)' // cnsreg doesn't save e(rss)		
		
			**Collect subsample RSS for linear model
		predict lin_resid_boot, res //collect residuals in variable
		replace lin_resid_boot = lin_resid_boot^2
		//collect RSS in of 3 different subsamples in 3 different variables
		egen lin_resid_boot1 = sum(lin_resid_boot) if quarter>=`period_1_start' & quarter<=`period_1_end'
		egen lin_resid_boot2 = sum(lin_resid_boot) if quarter>=`period_2_start' & quarter<=`period_2_end' 
		egen lin_resid_boot3 = sum(lin_resid_boot) if quarter>=`period_3_start' & quarter<=`period_3_end'
		forval i=1/3 { //RSS = max = mean = min for each variable ==> place number into a local
			sum lin_resid_boot`i'
			loc F`i'_lin_`inflvar'_boot `=r(mean)'
		}	
		drop lin_resid_boot*
		
		* create SSR using resampled y & gammastar
		// using gamma-dependent variables defined earlier
		
		// Compute fitted values and residuals, taking threshold as given
		cnsreg `inflvar'_boot `expvars' *_left *_right `controls' ,  constraints(1-2) nocons collinear		
		*local rss_h0 `=(e(rmse)^2)*e(df_r)' // rss for est model
		
		* initialize SSR & gamma scalars
		scalar rm_`inflvar'_boot = 100000
		scalar rm_`inflvar'_linboot = 100000
		scalar gl_`inflvar'_boot = -666
		scalar gu_`inflvar'_boot = -666
		scalar bL_boot = -666
		scalar bR_boot = -666
				
		qui sum `threshvar' , detail 
		qui levelsof `threshvar' , local(u_local)
		
		* Grid search using resampled y
		// Create constant
		cap drop ones
		gen ones = 1 
		
		local gamma_star_count_2 1
		foreach gamma_star in `u_local' {
		if (`gamma_star' >= thresh_`percentile') & (`gamma_star' <= thresh_`=100-`percentile'')  { //OB bounds for threshold values (adjustable by percentile)
		
				
				constraint drop 2 3
				di "Counter ....."
				di `gamma_star_count'
			
				
				preserve
				
				cap drop *left *right
				
				
				gen left = .
				replace left = 1 if `threshvar'<=`gamma_star'
				replace left = 0 if mi(left)

				gen right = .
				replace right = 1 if !(`threshvar'<=`gamma_star')
				replace right = 0 if mi(right)
					
				* create constant & urate (contemp & lagged diff) for each state
				gen ones_left = ones*left
				gen ones_right = ones*right
		
				
				foreach vv in `threshvar' `diffur' {
					gen `vv'_left = `vv'*left
					gen `vv'_right = `vv'*right
					if "`vv'"!="`threshvar'" { //for taking the mean 
						local diffur_split `diffur_split' `vv'_left `vv'_right
					}		
				}
					
				* Define constraint
				constraint 2 `gamma_star'*(`threshvar'_left-`threshvar'_right) + (ones_left-ones_right) = 0
				cnsreg `inflvar'_boot `expvars' *_left *_right `controls' ,  constraints(1-2) nocons collinear
				
				// Collect slope estimates
				scalar bL_2 = _b[`threshvar'_left]
				scalar bR_2 = _b[`threshvar'_right]				
				
				// Minimization for cnsreg; save F stat, gamma and slopes -- null is a threshold model
				if ((e(rmse)^2)*e(df_r))<rm_`inflvar'_boot { //cnsreg doesn't report e(rss), so it must be calculated as (root mean squared error)^2 * (residual degrees of freedom)
					
					scalar bL_boot = bL_2
					scalar bR_boot = bR_2
					scalar rm_`inflvar'_boot =((e(rmse)^2)*e(df_r))
					scalar gstar_`inflvar'_boot = `gamma_star'					
					local counter_`inflvar'  "`gamma_star_count_2'"
					noi di as text "`counter_`inflvar''"
					
					* change variables to reflect estimated gamma star case, then replace
					qui ds *left *right
					foreach vv in `r(varlist)' {
						gen `vv'_keep = `vv'
					}
					
					drop *left *right
				
					gen left = .
					replace left = 1 if `threshvar'<=`gstar'
					replace left = 0 if mi(left)

					gen right = .
					replace right = 1 if !(`threshvar'<=`gstar')
					replace right = 0 if mi(right)
					
					* create constant & urate (contemp & lagged diff) for each state
					gen ones_left = ones*left
					gen ones_right = ones*right
					
					foreach vv in `threshvar' {
						gen `vv'_left = `vv'*left
						gen `vv'_right = `vv'*right	
					}					
					
					predict e_null, resid
					summ e_null
					loc rss_h0 = `r(Var)'*`obs_n'
					local F_boot  `=`obs_n'*(`rss_h0' - rm_`inflvar'_boot)/rm_`inflvar'_boot'
					
					drop *left *right
					ren *_keep *
					
				}
				else {
					di ((e(rmse)^2)*e(df_r))					
				}

				cnsreg `inflvar'_linboot `expvars' *_left *_right `controls' ,  constraints(1-2) nocons collinear			
				
				// Null is a linear model
				if ((e(rmse)^2)*e(df_r))<rm_`inflvar'_linboot {
					scalar rm_`inflvar'_linboot =((e(rmse)^2)*e(df_r))
					local F_linboot  `=`obs_n'*(`F_lin_`inflvar'_boot' - rm_`inflvar'_linboot)/rm_`inflvar'_linboot'
					
					*F-statistic for subsample periods
					predict thresh_resid_boot, res //collect residuals in variable
					replace thresh_resid_boot = thresh_resid_boot^2
					//collect RSS in of 3 different subsamples in 3 different variables
					egen thresh_resid_boot1 = sum(thresh_resid_boot) if quarter>=`period_1_start' & quarter<=`period_1_end'
					egen thresh_resid_boot2 = sum(thresh_resid_boot) if quarter>=`period_2_start' & quarter<=`period_2_end' 
					egen thresh_resid_boot3 = sum(thresh_resid_boot) if quarter>=`period_3_start' & quarter<=`period_3_end'
					forval i=1/3 { //RSS = max = mean = min for each variable ==> place number into a local
						sum thresh_resid_boot`i'
						loc rm_`inflvar'_sub_boot`i' `=r(mean)'
					}
					local F1_linboot  `=`obs_n1_`inflvar''*(`F1_lin_`inflvar'_boot' - `rm_`inflvar'_sub_boot1')/`rm_`inflvar'_sub_boot1''
					local F2_linboot  `=`obs_n2_`inflvar''*(`F2_lin_`inflvar'_boot' - `rm_`inflvar'_sub_boot2')/`rm_`inflvar'_sub_boot2''
					local F3_linboot  `=`obs_n3_`inflvar''*(`F3_lin_`inflvar'_boot' - `rm_`inflvar'_sub_boot3')/`rm_`inflvar'_sub_boot3''
					drop thresh_resid_boot*
				}
				else {
					di ((e(rmse)^2)*e(df_r))					
				}				
				
				// drop coefficients & re-estimate
				restore
				}
				
			local gamma_star_count_2 `=`gamma_star_count_2'+1' //iteration counter	
		} //==> gamma_star loop
		
		* Save the F-stats, slopes and gammas from bootstrapping 
		matrix Fboot[`s',1] = `F_boot'
		matrix Fboot_lin[`s',1] = `F_linboot'
		matrix F1boot_lin[`s',1]= `F1_linboot' //subsample: period1 
		matrix F2boot_lin[`s',1]= `F2_linboot'  //subsample: period2 
		matrix F3boot_lin[`s',1]= `F3_linboot' //subsample: period3
		matrix gammaBoot[`s',1] = gstar_`inflvar'_boot
		matrix beta1Boot[`s',1] =bL_boot
		matrix beta2Boot[`s',1] =bR_boot
		
} // over bootstrap draws


*Generate bootstrapped CIs
preserve
foreach mm in Fboot Fboot_lin F1boot_lin F2boot_lin F3boot_lin gammaBoot beta1Boot beta2Boot {
	svmat `mm', names(`mm')
}

// Create p-values for existence of threshold
count if Fboot_lin1 >= `F_est' & !mi(Fboot_lin1)
scalar pval = `r(N)'/`n_sim'
histogram Fboot_lin1

// Subsample 1: p-value
count if F1boot_lin1 >= `F1_est' & !mi(F1boot_lin1)
scalar pval1 = `r(N)'/`n_sim'
histogram F1boot_lin1

// Subsample 2: p-value
count if F2boot_lin1 >= `F2_est' & !mi(F2boot_lin1)
scalar pval2 = `r(N)'/`n_sim'
histogram F2boot_lin1

// Subsample 3: p-value
count if F3boot_lin1 >= `F3_est' & !mi(F3boot_lin1)
scalar pval3 = `r(N)'/`n_sim'
histogram F3boot_lin1

// create CIs for slopes
gen delta_1 = abs(beta1Boot1-`beta1_star')
gen delta_2 = abs(beta2Boot1-`beta2_star')

forval i=1/2 {
	qui summ delta_`i', d
	loc q_`i' = `r(p95)'
	scalar beta`i'_Lci = `beta`i'_star' - `q_`i''
	scalar beta`i'_Uci = `beta`i'_star' + `q_`i''
}

// for gamma
summ Fboot1, d
// keep gammas for which the F stat is below the critical value
loc crit_F = `r(p95)'
gen gCI_set = gammaBoot1 if Fboot1 <= `crit_F'
summ gCI_set, d
scalar gamma_Lci = `r(min)'
scalar gamma_Uci = `r(max)'

line Fboot1 gammaBoot1, yli(`crit_F') sort
restore
} // end output suppression
}
